package com.dsk.flink.dsc.common.sink;

import cn.hutool.core.date.DateUtil;
import com.alibaba.druid.pool.DruidDataSource;
import com.dsk.flink.dsc.utils.EnvProperties;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.sink.RichSinkFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.util.concurrent.*;

public class MysqlDataTransferSink extends RichSinkFunction<String> {

    static Logger logger = LoggerFactory.getLogger(MysqlDataTransferSink.class);
    EnvProperties envProps;
    ExecutorService executorService;
    DruidDataSource dataSource;

    public MysqlDataTransferSink(EnvProperties envProps) {
        this.envProps = envProps;
    }

    @Override
    public void open(Configuration parameters) throws Exception {

        executorService = new ThreadPoolExecutor(4, 4, 20, TimeUnit.MINUTES, new LinkedBlockingDeque<>());
        //初始化获取配置
        String configTidbUrl = String.format(envProps.getDb_url(), envProps.getDb_host(), envProps.getDb_port(), envProps.getDb_database());
        dataSource = new DruidDataSource();
        dataSource.setDriverClassName("com.mysql.cj.jdbc.Driver");
        dataSource.setUsername(envProps.getDb_username());
        dataSource.setPassword(envProps.getDb_password());
        dataSource.setUrl(configTidbUrl);
        dataSource.setMaxActive(30);
        dataSource.setInitialSize(10);
        dataSource.setTestWhileIdle(true);
        dataSource.setMaxWait(20000);
        dataSource.setValidationQuery("select 1");
    }

    @Override
    public void close() throws Exception {
        executorService.shutdown();
        dataSource.close();
    }

    @Override
    public void invoke(String value, Context context) throws Exception {
        executorService.execute(() -> {
            try {
                executeSql(value);
            }catch (Exception e){
                e.printStackTrace();
            }
        });
    }

    private void executeSql(String sql) throws Exception{
        Connection connection = null;
        PreparedStatement pt = null;
        try {
            connection = dataSource.getConnection();
            pt = connection.prepareStatement(sql);
            pt.execute();
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("------错误时间:{}-----,sql:{}--------异常:{}", DateUtil.now(),sql,e.getMessage());
        } finally {
            if (pt != null) {
                pt.close();
            }
            if (connection != null) {
                connection.close();
            }
        }
    }
}
